@@ -145,8 +145,9 @@ confusionMatrix.default <- function(data, reference,
                                     ...) {
   if(!(mode %in% c("sens_spec", "prec_recall", "everything")))
     stop("`mode` should be either 'sens_spec', 'prec_recall', or 'everything'")
-  if(!is.factor(data)) data <- factor(data)
-  if(!is.factor(reference)) reference <- factor(reference)
+  if(!is.factor(data) | !is.factor(reference)) {
+    stop("`data` and `reference` should be factors with the same levels.", call. = FALSE)
+  }
   if(!is.character(positive) & !is.null(positive)) stop("positive argument must be character")
 
   if(length(levels(data)) > length(levels(reference)))
@@ -214,16 +215,24 @@ confusionMatrix.table <- function(data, positive = NULL,
     stop("with >2 classes, the prevalence vector must have names")
 
   propCI <- function(x) {
-    binom.test(sum(diag(x)), sum(x))$conf.int
+    res <- try(binom.test(sum(diag(x)), sum(x))$conf.int, silent = TRUE)
+    if(inherits(res, "try-error"))
+      res <- rep(NA, 2)
+    res
   }
 
   propTest <- function(x){
-    out <- binom.test(sum(diag(x)),
-                      sum(x),
-                      p = max(apply(x, 2, sum)/sum(x)),
-                      alternative = "greater")
-    unlist(out[c("null.value", "p.value")])
-
+    res <- try(
+      binom.test(sum(diag(x)),
+                 sum(x),
+                 p = max(apply(x, 2, sum)/sum(x)),
+                 alternative = "greater"),
+      silent = TRUE)
+    res <- if(inherits(res, "try-error"))
+      c("null.value.probability of success" = NA, p.value = NA)
+    else
+      res <- unlist(res[c("null.value", "p.value")])
+    res
   }
 
   overall <- c(unlist(e1071::classAgreement(data))[c("diag", "kappa")],
@@ -650,6 +659,7 @@ print.confusionMatrix <- function(x, mode = x$mode, digits = max(3, getOption("d
                      paste(overall[c("AccuracyNull", "AccuracyPValue")]),
                      "",
                      paste(overall["Kappa"]),
+                     "",
                      paste(overall["McnemarPValue"]))
 
     overallNames <- c("Accuracy", "95% CI",
@@ -657,6 +667,7 @@ print.confusionMatrix <- function(x, mode = x$mode, digits = max(3, getOption("d
                       "P-Value [Acc > NIR]",
                       "",
                       "Kappa",
+                      "",
                       "Mcnemar's Test P-Value")
 
     if(dim(x$table)[1] > 2){
