@@ -1,42 +1,53 @@
 #' @rdname caret-internal
 #' @export
-sbfIter <- function(x, y,
-                    testX, testY,
-                    sbfControl = sbfControl(), ...)
-{
-  if(is.null(colnames(x))) stop("x must have column names")
+sbfIter <- function(x, y, testX, testY, testPerf = NULL,
+                    sbfControl = sbfControl(), ...) {
+  if (is.null(colnames(x)))
+    stop("x must have column names")
 
-  if(is.null(testX) | is.null(testY)) stop("a test set must be specified")
+  if (is.null(testX) |
+      is.null(testY))
+    stop("a test set must be specified")
 
-  if(sbfControl$multivariate) {
+  if (sbfControl$multivariate) {
     scores <- sbfControl$functions$score(x, y)
-    if(length(scores) != ncol(x))
-      stop(paste("when control$multivariate == TRUE, 'scores'",
-                 "should return a vector with", ncol(x), "numeric values"))
-    } else  {
-      scores <- apply(x, 2, sbfControl$functions$score, y = y)
-    }
+    if (length(scores) != ncol(x))
+      stop(
+        paste(
+          "when control$multivariate == TRUE, 'scores'",
+          "should return a vector with",
+          ncol(x),
+          "numeric values"
+        )
+      )
+  } else  {
+    scores <- vapply(x, sbfControl$functions$score, double(1), y = y)
+  }
 
     retained <- sbfControl$functions$filter(scores, x, y)
     ## deal with zero length results
 
     testX <- testX[, which(retained), drop = FALSE]
 
-    fitObject <- sbfControl$functions$fit(x[, which(retained), drop = FALSE],
-                                          y,
-                                          ...)
+    fitObject <-
+      sbfControl$functions$fit(
+        x = x[, which(retained), drop = FALSE],
+        y = y,
+        ...
+      )
 
     modelPred <- sbfControl$functions$pred(fitObject, testX)
-    if(is.data.frame(modelPred) | is.matrix(modelPred))
-    {
-      if(is.matrix(modelPred)) modelPred <- as.data.frame(modelPred)
+    if (is.data.frame(modelPred) | is.matrix(modelPred)) {
+      if (is.matrix(modelPred))
+        modelPred <- as.data.frame(modelPred)
       modelPred$obs <- testY
-    } else modelPred <- data.frame(pred = modelPred, obs = testY)
-
+    } else
+      modelPred <- data.frame(pred = modelPred, obs = testY)
+    if (!is.null(testPerf))
+      modelPred <- cbind(modelPred, testPerf)
 
     list(variables = names(retained)[which(retained)],
          pred = modelPred)
-
   }
 
 
@@ -75,7 +86,7 @@ sbfIter <- function(x, y,
 #'
 #' @aliases sbf sbf.default sbf.formula predict.sbf
 #' @param x a data frame containing training data where samples are in rows and
-#' features are in columns.
+#' features are in columns. For the recipes method, \code{x} is a recipe object.
 #' @param y a numeric or factor vector containing the outcome for each sample.
 #' @param form A formula of the form \code{y ~ x1 + x2 + ...}
 #' @param data Data frame from which variables specified in \code{formula} are
@@ -161,15 +172,15 @@ sbf <- function (x, ...) UseMethod("sbf")
 #' @export
 "sbf.default" <-
   function(x, y,
-           sbfControl = sbfControl(), ...)
-  {
+           sbfControl = sbfControl(), ...) {
     startTime <- proc.time()
     funcCall <- match.call(expand.dots = TRUE)
 
     numFeat <- ncol(x)
     classLevels <- levels(y)
 
-    if(sbfControl$method == "oob") stop("out-of-bag resampling cannot be used with this function")
+    if (sbfControl$method == "oob")
+      stop("out-of-bag resampling cannot be used with this function")
 
     if(is.null(sbfControl$index)) sbfControl$index <- switch(
       tolower(sbfControl$method),
@@ -180,7 +191,8 @@ sbf <- function (x, ...) UseMethod("sbf")
       test = createDataPartition(y, 1, sbfControl$p),
       lgocv = createDataPartition(y, sbfControl$number, sbfControl$p))
 
-    if(is.null(names(sbfControl$index))) names(sbfControl$index) <- prettySeq(sbfControl$index)
+    if(is.null(names(sbfControl$index)))
+      names(sbfControl$index) <- prettySeq(sbfControl$index)
     if(is.null(sbfControl$indexOut)){
       sbfControl$indexOut <- lapply(sbfControl$index,
                                     function(training, allSamples) allSamples[-unique(training)],
@@ -192,20 +204,18 @@ sbf <- function (x, ...) UseMethod("sbf")
                              obs = sample(y, min(10, length(y))))
 
     if(is.factor(y))
-    {
-      for(i in seq(along = classLevels)) testOutput[, classLevels[i]] <- runif(nrow(testOutput))
-    }
+      for(i in seq(along = classLevels))
+        testOutput[, classLevels[i]] <- runif(nrow(testOutput))
+
 
     test <- sbfControl$functions$summary(testOutput, lev = classLevels)
     perfNames <- names(test)
 
     ## Set or check the seeds when needed
-    if(is.null(sbfControl$seeds))
-    {
+    if(is.null(sbfControl$seeds)) {
       sbfControl$seeds <- sample.int(n = 1000000, size = length(sbfControl$index) + 1)
     } else {
-      if(!(length(sbfControl$seeds) == 1 && is.na(sbfControl$seeds)))
-      {
+      if(!(length(sbfControl$seeds) == 1 && is.na(sbfControl$seeds))) {
         if(length(sbfControl$seeds) != length(sbfControl$index) + 1)
           stop(paste("Bad seeds: the seed object should be an integer vector of length",
                      length(sbfControl$index) + 1))
@@ -216,9 +226,7 @@ sbf <- function (x, ...) UseMethod("sbf")
 
     #########################################################################
 
-
-    if(sbfControl$method == "LOOCV")
-    {
+    if(sbfControl$method == "LOOCV") {
       tmp <- looSbfWorkflow(x = x, y = y, ppOpts = preProcess,
                             ctrl = sbfControl, lev = classLevels, ...)
       resamples <- do.call("rbind", tmp$everything[names(tmp$everything) == "pred"])
@@ -248,17 +256,18 @@ sbf <- function (x, ...) UseMethod("sbf")
     retained <- sbfControl$functions$filter(scores, x, y)
 
     finalTime <- system.time(
-      fit <- sbfControl$functions$fit(x[, retained, drop = FALSE],
-                                      y,
-                                      ...))
-
-
+      fit <-
+        sbfControl$functions$fit(
+          x[, retained, drop = FALSE],
+          y,
+          ...
+        )
+    )
 
     performance <- data.frame(t(performance))
     performance <- performance[,!grepl("\\.cell|Resample", colnames(performance))]
 
-    if(is.factor(y) & any(names(resamples) == ".cell1"))
-    {
+    if(is.factor(y) & any(names(resamples) == ".cell1")) {
       keepers <- c("Resample", grep("\\.cell", names(resamples), value = TRUE))
       resampledCM <- resamples[,keepers]
       resamples <- resamples[, -grep("\\.cell", names(resamples))]
@@ -292,9 +301,11 @@ sbf <- function (x, ...) UseMethod("sbf")
         obsLevels = classLevels,
         dots = list(...)),
       class = "sbf")
-    if(sbfControl$timingSamps > 0)
-    {
-      out$times$prediction <- system.time(predict(out, x[1:min(nrow(x), sbfControl$timingSamps),,drop = FALSE]))
+    if(sbfControl$timingSamps > 0) {
+      out$times$prediction <-
+        system.time(
+          predict(out, x[1:min(nrow(x), sbfControl$timingSamps),,drop = FALSE])
+        )
     } else  out$times$prediction <- rep(NA, 3)
     out
   }
@@ -302,8 +313,7 @@ sbf <- function (x, ...) UseMethod("sbf")
 #' @rdname sbf
 #' @importFrom stats .getXlevels contrasts model.matrix model.response
 #' @export
-sbf.formula <- function (form, data, ..., subset, na.action, contrasts = NULL)
-{
+sbf.formula <- function (form, data, ..., subset, na.action, contrasts = NULL) {
   m <- match.call(expand.dots = FALSE)
   if (is.matrix(eval.parent(m$data))) m$data <- as.data.frame(data)
   m$... <- m$contrasts <- NULL
@@ -326,12 +336,378 @@ sbf.formula <- function (form, data, ..., subset, na.action, contrasts = NULL)
   res
 }
 
+
 ######################################################################
+
+#' @rdname sbf
+#' @export
+"sbf.recipe" <-
+  function(x, data,
+           sbfControl = sbfControl(), ...) {
+    startTime <- proc.time()
+    funcCall <- match.call(expand.dots = TRUE)
+
+    orig_rec <- x
+    trained_rec <- prep(
+      x, training = data,
+      fresh = TRUE,
+      retain = TRUE,
+      verbose = FALSE,
+      stringsAsFactors = TRUE
+    )
+    x <- juice(trained_rec, all_predictors(), composition = "data.frame")
+    y <- juice(trained_rec, all_outcomes(), composition = "data.frame")
+    if(ncol(y) > 1)
+      stop("`safs` doesn't support multivariate outcomes", call. = FALSE)
+    y <- y[[1]]
+    is_weight <- summary(trained_rec)$role == "case weight"
+    if(any(is_weight))
+      stop("`safs` does not allow for weights.", call. = FALSE)
+
+    is_perf <- summary(trained_rec)$role == "performance var"
+    if(any(is_perf)) {
+      perf_data <- juice(trained_rec, has_role("performance var"))
+    } else perf_data <- NULL
+
+    numFeat <- ncol(x)
+    classLevels <- levels(y)
+
+    if (sbfControl$method == "oob")
+      stop("out-of-bag resampling cannot be used with this function")
+
+    if(is.null(sbfControl$index)) sbfControl$index <- switch(
+      tolower(sbfControl$method),
+      cv = createFolds(y, sbfControl$number, returnTrain = TRUE),
+      repeatedcv = createMultiFolds(y, sbfControl$number, sbfControl$repeats),
+      loocv = createFolds(y, length(y), returnTrain = TRUE),
+      boot =, boot632 = createResample(y, sbfControl$number),
+      test = createDataPartition(y, 1, sbfControl$p),
+      lgocv = createDataPartition(y, sbfControl$number, sbfControl$p))
+
+    if(is.null(names(sbfControl$index)))
+      names(sbfControl$index) <- prettySeq(sbfControl$index)
+    if(is.null(sbfControl$indexOut)){
+      sbfControl$indexOut <- lapply(sbfControl$index,
+                                    function(training, allSamples) allSamples[-unique(training)],
+                                    allSamples = seq(along = y))
+      names(sbfControl$indexOut) <- prettySeq(sbfControl$indexOut)
+    }
+    ## check summary function and metric
+    testOutput <- data.frame(pred = sample(y, min(10, length(y))),
+                             obs = sample(y, min(10, length(y))))
+
+    if(is.factor(y))
+      for(i in seq(along = classLevels))
+        testOutput[, classLevels[i]] <- runif(nrow(testOutput))
+    if(!is.null(perf_data))
+      testOutput <- cbind(
+        testOutput,
+        perf_data[sample(1:nrow(perf_data), nrow(testOutput)),, drop = FALSE]
+      )
+
+    test <- sbfControl$functions$summary(testOutput, lev = classLevels)
+    perfNames <- names(test)
+
+    ## Set or check the seeds when needed
+    if(is.null(sbfControl$seeds)) {
+      sbfControl$seeds <- sample.int(n = 1000000, size = length(sbfControl$index) + 1)
+    } else {
+      if(!(length(sbfControl$seeds) == 1 && is.na(sbfControl$seeds))) {
+        if(length(sbfControl$seeds) != length(sbfControl$index) + 1)
+          stop(paste("Bad seeds: the seed object should be an integer vector of length",
+                     length(sbfControl$index) + 1))
+      }
+    }
+
+
+
+    #########################################################################
+
+    if(sbfControl$method == "LOOCV") {
+      tmp <- sbf_loo_rec(rec = orig_rec, data = data,
+                         ctrl = sbfControl, lev = classLevels, ...)
+      resamples <- do.call("rbind", tmp$everything[names(tmp$everything) == "pred"])
+      rownames(resamples) <- 1:nrow(resamples)
+      selectedVars <- tmp$everything[names(tmp$everything) == "variables"]
+      performance <- tmp$performance
+    } else {
+      tmp <- sbf_rec(rec = orig_rec, data = data,
+                     ctrl = sbfControl, lev = classLevels, ...)
+      resamples <- do.call("rbind", tmp$everything[names(tmp$everything) == "resamples"])
+      rownames(resamples) <- 1:nrow(resamples)
+      selectedVars <- tmp$everything[names(tmp$everything) == "selectedVars"]
+      performance <- tmp$performance
+    }
+
+    #########################################################################
+
+    varList <- unique(unlist(selectedVars))
+    if(sbfControl$multivariate) {
+      scores <- sbfControl$functions$score(x, y)
+      if(length(scores) != ncol(x))
+        stop(paste("when control$multivariate == TRUE, 'scores'",
+                   "should return a vector with", ncol(x), "numeric values"))
+    } else  {
+      scores <- apply(x, 2, sbfControl$functions$score, y = y)
+    }
+    retained <- sbfControl$functions$filter(scores, x, y)
+
+    finalTime <- system.time(
+      fit <-
+        sbfControl$functions$fit(
+          x = x[, retained, drop = FALSE],
+          y = y,
+          ...
+        )
+    )
+
+    performance <- data.frame(t(performance))
+    performance <- performance[,!grepl("\\.cell|Resample", colnames(performance))]
+
+    if(is.factor(y) & any(names(resamples) == ".cell1")) {
+      keepers <- c("Resample", grep("\\.cell", names(resamples), value = TRUE))
+      resampledCM <- resamples[,keepers]
+      resamples <- resamples[, -grep("\\.cell", names(resamples))]
+    } else resampledCM <- NULL
+
+    resamples <- switch(sbfControl$returnResamp,
+                        none = NULL,
+                        all =, final = resamples)
+
+    endTime <- proc.time()
+    times <- list(everything = endTime - startTime,
+                  final = finalTime)
+
+    #########################################################################
+    ## Now, based on probability or static ranking, figure out the best vars
+    ## and the best subset size and fit final model
+
+    out <- structure(
+      list(
+        pred = if(sbfControl$saveDetails) tmp else NULL,
+        variables = selectedVars,
+        results = performance,
+        fit = fit,
+        optVariables = names(retained)[retained],
+        call = funcCall,
+        control = sbfControl,
+        resample = resamples,
+        metrics = perfNames,
+        times = times,
+        resampledCM = resampledCM,
+        obsLevels = classLevels,
+        dots = list(...),
+        recipe = trained_rec),
+      class = "sbf")
+    if(sbfControl$timingSamps > 0) {
+      out$times$prediction <-
+        system.time(
+          predict(out, x[1:min(nrow(x), sbfControl$timingSamps),,drop = FALSE])
+        )
+    } else  out$times$prediction <- rep(NA, 3)
+    out
+  }
+
+
+#' @import foreach
+sbf_rec <- function(rec, data, ctrl, lev, ...) {
+  loadNamespace("caret")
+
+
+  resampleIndex <- ctrl$index
+  if(ctrl$method %in% c("boot632")){
+    resampleIndex <- c(list("AllData" = rep(0, nrow(x))), resampleIndex)
+    ctrl$indexOut <- c(list("AllData" = rep(0, nrow(x))),  ctrl$indexOut)
+  }
+
+  `%op%` <- getOper(ctrl$allowParallel && getDoParWorkers() > 1)
+  result <- foreach(
+    iter = seq(along = resampleIndex),
+    .combine = "c",
+    .verbose = FALSE,
+    .errorhandling = "stop",
+    .packages = c("caret", "recipes")) %op% {
+      if (!(length(ctrl$seeds) == 1 && is.na(ctrl$seeds)))
+        set.seed(ctrl$seeds[iter])
+
+      loadNamespace("caret")
+      requireNamespaceQuietStop("methods")
+
+      if(names(resampleIndex)[iter] != "AllData") {
+        modelIndex <- resampleIndex[[iter]]
+        holdoutIndex <- ctrl$indexOut[[iter]]
+      } else {
+        modelIndex <- 1:nrow(data)
+        holdoutIndex <- modelIndex
+      }
+
+      # reprocess recipe
+      resampled_rec <- prep(
+        rec,
+        training = data[modelIndex, ],
+        fresh = TRUE,
+        retain = TRUE,
+        verbose = FALSE,
+        stringsAsFactors = TRUE
+      )
+      x_tr <- juice(resampled_rec, all_predictors(), composition = "data.frame")
+      y_tr <- juice(resampled_rec, all_outcomes(), composition = "data.frame")
+      y_tr <- y_tr[[1]]
+      x_te <- bake(resampled_rec, new_data = data[ holdoutIndex, ],
+                   all_predictors(), composition = "data.frame")
+      y_te <- bake(resampled_rec, new_data = data[ holdoutIndex, ],
+                   all_outcomes(), composition = "data.frame")
+      y_te <- y_te[[1]]
+      is_perf <- summary(resampled_rec)$role == "performance var"
+      if(any(is_perf)) {
+        perf_tr <- juice(resampled_rec, has_role("performance var"))
+        perf_te <- bake(
+          resampled_rec,
+          new_data = data[ holdoutIndex, ],
+          has_role("performance var")
+        )
+      } else {
+        perf_tr <- NULL
+        perf_te <- NULL
+      }
+
+      sbfResults <- sbfIter(x = x_tr,
+                            y = y_tr,
+                            testX = x_te,
+                            testY = y_te,
+                            testPerf = perf_te,
+                            sbfControl = ctrl,
+                            ...)
+      if(ctrl$saveDetails) {
+        tmpPred <- sbfResults$pred
+        tmpPred$Resample <- names(resampleIndex)[iter]
+        tmpPred$rowIndex <- (1:nrow(data))[unique(holdoutIndex)]
+      } else tmpPred <- NULL
+      resamples <- ctrl$functions$summary(sbfResults$pred, lev = lev)
+      if(is.factor(y_tr) && length(lev) <= 50)
+        resamples <- c(resamples, flatTable(sbfResults$pred$pred, sbfResults$pred$obs))
+      resamples <- data.frame(t(resamples))
+      resamples$Resample <- names(resampleIndex)[iter]
+
+      list(resamples = resamples, selectedVars = sbfResults$variables, pred = tmpPred)
+    }
+
+  resamples <- rbind.fill(result[names(result) == "resamples"])
+  pred <- if(ctrl$saveDetails) rbind.fill(result[names(result) == "pred"]) else NULL
+  performance <- MeanSD(resamples[,!grepl("Resample", colnames(resamples)),drop = FALSE])
+
+  if(ctrl$method %in% c("boot632")) {
+    modelIndex <- 1:nrow(x)
+    holdoutIndex <- modelIndex
+    appResults <- sbfIter(x = x_tr,
+                          y = y_tr,
+                          testX = x_te,
+                          testY = y_te,
+                          testPerf = perf_te,
+                          ctrl,
+                          ...)
+    apparent <- ctrl$functions$summary(appResults$pred, lev = lev)
+    perfNames <- names(apparent)
+    perfNames <- perfNames[perfNames != "Resample"]
+
+    const <- 1-exp(-1)
+
+    for(p in seq(along = perfNames))
+      performance[perfNames[p]] <-
+      (const * performance[perfNames[p]]) +  ((1-const) * apparent[perfNames[p]])
+  }
+
+  list(
+    performance = performance,
+    everything = result,
+    predictions = if (ctrl$saveDetails)
+      pred
+    else
+      NULL
+  )
+}
+
+
+sbf_loo_rec <- function(rec, data, ctrl, lev, ...) {
+  loadNamespace("caret")
+
+  resampleIndex <- ctrl$index
+
+  vars <- vector(mode = "list", length = nrow(data))
+
+  `%op%` <- getOper(ctrl$allowParallel && getDoParWorkers() > 1)
+  result <- foreach(
+    iter = seq(along = resampleIndex),
+    .combine = "c",
+    .verbose = FALSE,
+    .errorhandling = "stop",
+    .packages = c("caret", "recipes")) %op% {
+
+      if(!(length(ctrl$seeds) == 1 && is.na(ctrl$seeds)))
+        set.seed(ctrl$seeds[iter])
+
+      loadNamespace("caret")
+      requireNamespaceQuietStop("methods")
+      modelIndex <- resampleIndex[[iter]]
+      holdoutIndex <- -unique(resampleIndex[[iter]])
+
+      # reprocess recipe
+      resampled_rec <- prep(
+        rec,
+        training = data[modelIndex, ],
+        fresh = TRUE,
+        retain = TRUE,
+        verbose = FALSE,
+        stringsAsFactors = TRUE
+      )
+      x_tr <- juice(resampled_rec, all_predictors(), composition = "data.frame")
+      y_tr <- juice(resampled_rec, all_outcomes(), composition = "data.frame")
+      y_tr <- y_tr[[1]]
+      x_te <- bake(resampled_rec, new_data = data[ holdoutIndex, ],
+                   all_predictors(), composition = "data.frame")
+      y_te <- bake(resampled_rec, new_data = data[ holdoutIndex, ],
+                   all_outcomes(), composition = "data.frame")
+      y_te <- y_te[[1]]
+      is_perf <- summary(resampled_rec)$role == "performance var"
+      if(any(is_perf)) {
+        perf_tr <- juice(resampled_rec, has_role("performance var"))
+        perf_te <- bake(
+          resampled_rec,
+          new_data = data[ holdoutIndex, ],
+          has_role("performance var")
+        )
+      } else {
+        perf_tr <- NULL
+        perf_te <- NULL
+      }
+
+      sbfResults <- sbfIter(x = x_tr,
+                            y = y_tr,
+                            testX = x_te,
+                            testY = y_te,
+                            testPerf = perf_te,
+                            sbfControl = ctrl,
+                            ...)
+
+      sbfResults
+    }
+  resamples <- do.call("rbind", result[names(result) == "pred"])
+  performance <- ctrl$functions$summary(resamples, lev = lev)
+
+  list(
+    performance = performance,
+    everything = result,
+    predictions = if (ctrl$saveDetails)
+      resamples
+    else
+      NULL
+  )
+}
+
 ######################################################################
 
 #' @export
-print.sbf <- function(x, top = 5, digits = max(3, getOption("digits") - 3), ...)
-{
+print.sbf <- function(x, top = 5, digits = max(3, getOption("digits") - 3), ...) {
 
   cat("\nSelection By Filter\n\n")
 
@@ -345,8 +721,7 @@ print.sbf <- function(x, top = 5, digits = max(3, getOption("digits") - 3), ...)
   print(format(x$results, digits = digits), row.names = FALSE)
   cat("\n")
 
-  if(length(x$optVariables) > 0)
-  {
+  if(length(x$optVariables) > 0) {
     cat("Using the training set, ",
         length(x$optVariables),
         ifelse(length(x$optVariables) > 1,
@@ -371,30 +746,31 @@ print.sbf <- function(x, top = 5, digits = max(3, getOption("digits") - 3), ...)
                    smallVars, "%)", sep = "")
   varText <- paste(varText, collapse = ", ")
 
-  if(!all(is.na(smallVars)))
-  {
-    cat("During resampling, the top ",
-        top,
-        " selected variables (out of a possible ",
-        length(vars),
-        "):\n   ",
-        varText,
-        "\n\n",
-        sep = "")
-    cat("On average, ",
-        round(mean(unlist(lapply(x$variables, length))), 1),
-        " variables were selected (min = ",
-        round(min(unlist(lapply(x$variables, length))), 1),
-        ", max = ",
-        round(max(unlist(lapply(x$variables, length))), 1),
-        ")\n",
-        sep = "")
+  if(!all(is.na(smallVars))) {
+    cat(
+      "During resampling, the top ",
+      top,
+      " selected variables (out of a possible ",
+      length(vars),
+      "):\n   ",
+      varText,
+      "\n\n",
+      sep = ""
+    )
+    cat(
+      "On average, ",
+      round(mean(unlist(lapply(x$variables, length))), 1),
+      " variables were selected (min = ",
+      round(min(unlist(lapply(x$variables, length))), 1),
+      ", max = ",
+      round(max(unlist(lapply(x$variables, length))), 1),
+      ")\n",
+      sep = ""
+    )
   } else {
     cat("During resampling, no variables were selected.\n\n")
   }
 
-
-
   invisible(x)
 }
 
@@ -403,24 +779,27 @@ print.sbf <- function(x, top = 5, digits = max(3, getOption("digits") - 3), ...)
 #' @rdname sbf
 #' @importFrom stats .checkMFClasses delete.response model.frame model.matrix na.omit
 #' @export
-predict.sbf <- function(object, newdata = NULL, ...)
-{
-  if(!is.null(newdata))
-  {
-    if (inherits(object, "sbf.formula"))
-    {
+predict.sbf <- function(object, newdata = NULL, ...) {
+  if (!is.null(newdata)) {
+    if (inherits(object, "sbf.formula")) {
       newdata <- as.data.frame(newdata)
       rn <- row.names(newdata)
       Terms <- delete.response(object$terms)
-      m <- model.frame(Terms, newdata, na.action = na.omit,
-                       xlev = object$xlevels)
-      if (!is.null(cl <- attr(Terms, "dataClasses"))) .checkMFClasses(cl, m)
+      m <- model.frame(Terms, newdata, na.action = na.omit, xlev = object$xlevels)
+      if (!is.null(cl <- attr(Terms, "dataClasses")))
+        .checkMFClasses(cl, m)
       keep <- match(row.names(m), rn)
       newdata <- model.matrix(Terms, m, contrasts = object$contrasts)
       xint <- match("(Intercept)", colnames(newdata), nomatch = 0)
-      if (xint > 0) newdata <- newdata[, -xint, drop = FALSE]
+      if (xint > 0)
+        newdata <- newdata[,-xint, drop = FALSE]
+    } else {
+      if (any(names(object) == "recipe") && !is.null(object$recipe)) {
+        newdata <-
+          bake(object$recipe, newdata, all_predictors(), composition = "data.frame")
+      }
     }
-    if(!all(object$optVariables %in% colnames(newdata)))
+    if (!all(object$optVariables %in% colnames(newdata)))
       stop("required columns in newdata are missing", call. = FALSE)
     newdata <- newdata[, object$optVariables, drop = FALSE]
     out <- object$control$functions$pred(object$fit, newdata)
